var conditionModule = angular.module('condition', ['wemServices', 'wemDirectives', 'i18n', 'ui.bootstrap', 'uiGmapgoogle-maps']);

conditionModule.factory('RecursionHelper', ['$compile', function ($compile) {
    return {
        /**
         * Manually compiles the element, fixing the recursion loop.
         * @param element
         * @param [link] A post-link function, or an object with function(s) registered via pre and post properties.
         * @returns An object containing the linking functions.
         */
        compile: function (element, link) {
            // Normalize the link parameter
            if (angular.isFunction(link)) {
                link = {post: link};
            }

            // Break the recursion loop by removing the contents
            var contents = element.contents().remove();
            var compiledContents;
            return {
                pre: (link && link.pre) ? link.pre : null,
                /**
                 * Compiles and re-adds the contents
                 */
                post: function (scope, element) {
                    // Compile the contents
                    if (!compiledContents) {
                        compiledContents = $compile(contents);
                    }
                    // Re-add the compiled contents to the element
                    compiledContents(scope, function (clone) {
                        element.append(clone);
                    });

                    // Call the post-linking function, if any
                    if (link && link.post) {
                        link.post.apply(null, arguments);
                    }
                }
            };
        }
    };
}]);

conditionModule.directive("condition", function (RecursionHelper) {
    return {
        restrict: "E",
        scope: {
            countlabel:'@',
            countlabeli18n:'@',
            disableEdition: '=',
            disableBoolean: '=',
            displayCount: '=',
            form: '=',
            itemType: '@',
            labelNew: '=',
            level: '@',
            parentIndex: '=',
            parentName: '=',
            parentRef: '=',
            ref: '=',
            showCount: '=',
            types: '='
        },
        templateUrl: ManagersContext.localTemplatesPath + "/condition/builder.html",

        compile: function (element) {
            // Use the compile function from the RecursionHelper,
            // And return the linking function(s) which it returns
            return RecursionHelper.compile(element);
        },
        controller: function ($scope, $rootScope, queryService, $modal, datepickerConfig, i18nService) {
            if(angular.isUndefined($scope.disableEdition)) {
                $scope.disableEdition = false;
            }
            if (angular.isUndefined($scope.disableBoolean)) {
                $scope.disableBoolean = false;
            }

            $scope.getCountLabel =  function(){
                if(!$scope.countlabeli18n){
                    if(!$scope.countlabel) {
                        return "";
                    } else {
                        return $scope.countlabel;
                    }
                } else {
                    return i18nService.message($scope.countlabeli18n);
                }
            };

            if(!$scope.level) {
                $scope.level = 0;
            }

            if (angular.isUndefined($scope.displayCount)) {
                $scope.displayCount = true;
            }

            if (angular.isUndefined($scope.labelNew)) {
                $scope.labelNew = "condition.new";
            }
            $scope.newConditionText = i18nService.message($scope.labelNew);
            if($scope.form && $scope.form.scoringId){
                $scope.scoringId = $scope.form.scoringId.$modelValue;
            }

            // date config
            datepickerConfig.startingDay = 1;
            datepickerConfig.minMode = "day";
            datepickerConfig.datepickerMode = "day";
            datepickerConfig.showWeeks = false;
            $scope.currentText = i18nService.message("serverSettings.manageWem.datepicker.today");
            $scope.closeText = i18nService.message("serverSettings.manageWem.datepicker.close");
            $scope.datePickers = {};

            $scope.openDatePopup = function ($event, selector) {
                $event.preventDefault();
                $event.stopPropagation();
                angular.forEach($scope.datePickers, function(value, key) {
                    $scope.datePickers[key] = false;
                });
                $scope.datePickers[selector] = true;
            };

            $scope.hasError = function(form, propertyName) {
                return ManagersUtils.hasErrors($scope.form, form, propertyName);
            };

            $scope.getHumanizeTime = function (value, unit) {
                return moment.duration(value, unit).humanize();
            };

            $scope.updateTemplate = function () {
                if ($scope.ref && $scope.ref.type) {
                    $scope.conditionType = $scope.getConditionType($scope.ref);
                    if ($scope.conditionType !== null) {
                        var conditionTypeId = $scope.conditionType.id;
                        var conditionTemplatesDir = ManagersContext.localTemplatesPath + '/condition/conditions/';
                        var propertyTemplatesDir = conditionTemplatesDir + 'property/';

                        // special handling of property templates
                        if (conditionTypeId.indexOf('Property') !== -1) {
                            $scope.conditionLabelKey = 'condition.propertyCondition.' + conditionTypeId;
                            $scope.conditionTemplateUrl = propertyTemplatesDir + 'propertyCondition.html';
                            $scope.conditionSelectTemplate = conditionTypeId == 'eventPropertyCondition' ? propertyTemplatesDir + conditionTypeId + 'Select.html'
                                : propertyTemplatesDir + 'defaultPropertyConditionSelect.html';
                        } else {
                            $scope.conditionTemplateUrl = conditionTemplatesDir + conditionTypeId + '.html';
                        }
                    }
                }
            };

            $scope.getConditionType = function (condition) {
                function resolveConditionTypeIn(list, condition) {
                    if (list) {
                        for (var i = 0; i < list.length; i++) {
                            var conditionType = list[i];
                            if (conditionType.id == condition.type) {
                                return conditionType;
                            }
                        }
                    }
                }

                if (condition.conditionType) {
                    return condition.conditionType;
                }
                var lists = [$scope.types, $scope.conditionTypes, $scope.eventConditionTypes];
                for (var i = 0; i < lists.length; i++) {
                    var resolved = resolveConditionTypeIn(lists[i], condition);
                    if (resolved) {
                        return resolved;
                    }
                }
                return null;
            };

            $scope.addCondition = function (conditionType, targetName, wrapIn) {
                if (wrapIn) {
                    var copyCurrentRef = angular.copy($scope.ref);
                    targetName = "subConditions";
                    $scope.ref = {
                        "type": "booleanCondition",
                        "parameterValues": {
                            "operator": wrapIn,
                            "subConditions": [copyCurrentRef]
                        }
                    };
                    if ($scope.parentRef) {
                        $scope.parentRef.parameterValues[$scope.parentName][$scope.parentIndex] = $scope.ref;
                    }
                }


                var newCondition = {
                    "type": conditionType.id,
                    "parameterValues": {}
                };
                for (var i = 0; i < conditionType.parameters.length; i++) {
                    var parameter = conditionType.parameters[i];
                    if (parameter.multivalued) {
                        if (parameter.defaultValue !== undefined && parameter.defaultValue != null && parameter.defaultValue.length > 0) {
                            newCondition.parameterValues[parameter.id] = parameter.defaultValue.split(",");
                        } else {
                            newCondition.parameterValues[parameter.id] = [];
                        }
                    } else {
                        if (parameter.defaultValue !== undefined && parameter.defaultValue != null) {
                            if (parameter.type == "integer") {
                                newCondition.parameterValues[parameter.id] = parseInt(parameter.defaultValue);
                            } else {
                                newCondition.parameterValues[parameter.id] = parameter.defaultValue;
                            }
                        }
                    }
                }

                if (targetName) {
                    if (Object.prototype.toString.call($scope.ref.parameterValues[targetName]) === '[object Array]') {
                        $scope.ref.parameterValues[targetName].push(newCondition);
                    } else {
                        $scope.ref.parameterValues[targetName] = newCondition;
                    }
                } else {
                    $scope.ref = newCondition;
                }
            };

            $scope.onRemove = function (parentName, index) {
                console.log("condition.js : onRemove(" + parentName + "," + index + ") called");
                var parent = $scope.ref.parameterValues[parentName];
                if (parent instanceof Array) {
                    parent.splice(index, 1);
                    $scope.ref.parameterValues[parentName] = parent;
                    if ($scope.ref.type == 'booleanCondition') {
                        if (parent.length == 1) {
                            $scope.ref = $scope.ref.parameterValues.subConditions[0]; // unwrap boolean condition
                            if ($scope.parentRef) {
                                $scope.parentRef.parameterValues[$scope.parentName][$scope.parentIndex] = $scope.ref;
                            }
                        } else if (parent.length == 0) {
                            // delete empty boolean condition
                            if ($scope.parentRef) {
                                parent = $scope.parentRef.parameterValues[$scope.parentName];
                                if (parent instanceof Array) {
                                    parent.splice($scope.parentIndex, 1);
                                    $scope.parentRef.parameterValues[$scope.parentName] = parent;
                                } else {
                                    $scope.parentRef.parameterValues[$scope.parentName] = null;
                                }
                            } else {
                                $scope.ref = {};
                            }
                        }
                    }
                } else {
                    $scope.ref.parameterValues[parentName] = null;
                }
            };

            $scope.onRemoveRoot = function () {
                console.log("condition.js : onRemoveRoot() called");
                $scope.ref = {};
            };

            $scope.onAddValue = function (values, newVal, $event) {
                // if we're dealing with dates, convert the given value to a date first
                if (values == $scope.ref.parameterValues.propertyValuesDate && typeof newVal === 'string') {
                    console.log("Condition builder : Dealing with date !");
                    newVal = new Date(newVal);
                }
                values.push(newVal);

                $event.preventDefault();
            };

            $scope.onRemoveValue = function (values, pos, $event) {
                values.splice(pos, 1);

                $event.preventDefault();
            };

            $scope.canWrapBoolCondition = function (operator) {
                return (!$scope.disableBoolean && $scope.ref.type != 'booleanCondition'
                && ($scope.parentRef == null || $scope.parentRef.type != 'booleanCondition' || $scope.parentRef.parameterValues.operator != operator));
            };

            $scope.openAddConditionModal = function (targetName, wrapIn) {
                if($scope.form && $scope.form.scoringType){
                    $scope.scoringType = $scope.form.scoringType.$viewValue;
                }
                var modalInstance = $modal.open({
                    templateUrl: ManagersContext.localTemplatesPath + '/condition/addModal.html',
                    controller: function ($scope, $modalInstance, conditionTypes, newConditionText) {
                        $scope.conditionTypes = _.filter(conditionTypes, function(value) {
                            return i18nService.message('condition.' + value.id + '.name').indexOf("???") == -1;
                        });
                        $scope.newConditionText = newConditionText;

                        $scope.ok = function () {
                            $modalInstance.close($scope.conditionType);
                        };
                    },
                    size: 'lg',
                    resolve: {
                        conditionTypes: function () {
                            var filteredTypes = $scope.types;
                            if($scope.scoringType){
                                if($scope.scoringType !== 'st:all'){
                                    filteredTypes = _.filter(filteredTypes, function (conditionType) {
                                        return $scope.scoringType.split(':')[1] == conditionType.profileTag;
                                    });
                                }else{
                                    filteredTypes = _.filter(filteredTypes, function (conditionType) {
                                        return _.contains($rootScope.allowedTags, conditionType.profileTag);
                                    });
                                }
                            }
                            return _.filter(filteredTypes, function (conditionType) {
                                return (conditionType.id != "booleanCondition" && conditionType.id != "notCondition" &&
                                conditionType.id != "matchAllCondition");
                            });
                        },
                        newConditionText: function() {
                            return $scope.newConditionText;
                        }
                    }
                });

                modalInstance.result.then(function (conditionType) {
                    if (conditionType) {
                        $scope.addCondition(conditionType, targetName, wrapIn);
                    }
                });
            };

            // init phase
            if (!$scope.ref) {
                console.log("condition.js: no condition passed, creating empty one.");
                $scope.ref = {};
            }

            console.log("condition.js: Initializing condition controller for condition " + $scope.ref + " of type=" + $scope.ref.type + " parentIndex=" + $scope.parentIndex);

            $scope.updateTemplate();
            $scope.$watch("ref", function (newValue, oldValue) {
                if (newValue != oldValue) {
                    console.log("condition.js : condition changed, refreshing...");
                    $scope.updateTemplate();
                }
            });

            if ($scope.itemType) {
                $scope.currentItemType = $scope.itemType;
            } else {
                $scope.currentItemType = $scope.$parent.currentItemType;
            }

            if (!$scope.showCount) {
                $scope.showCount = $scope.$parent.showCount;
            }

            $scope.validateCondition = function(condition) {
                if (!condition || !condition.type || !condition.parameterValues) {
                    return false;
                }

                for (var parameter in condition.parameterValues) {
                    if (condition.parameterValues.hasOwnProperty(parameter)) {
                        if (condition.parameterValues[parameter] == undefined) {
                            return false;
                        } else if (condition.parameterValues[parameter].constructor == Array
                            && condition.parameterValues[parameter][0] == undefined) { // multiple parameters are initialized
                            return false;
                        }
                    }
                }

                if (condition.type == 'booleanCondition') {
                    if (condition.parameterValues.subConditions.length == 0) {
                        return false;
                    }
                    for (var c = 0; c < condition.parameterValues.subConditions.length; c++) {
                        var subCondition = condition.parameterValues.subConditions[c];
                        var result = $scope.validateCondition(subCondition);
                        if (!result) {
                            return false;
                        }
                    }
                }

                if (condition.type == 'pastEventCondition') {
                    var result = $scope.validateCondition(condition.parameterValues.eventCondition);
                    if (!result) {
                        return false;
                    }
                }

                if (condition.type == 'campaignEngagedCondition') {
                    if (!condition.parameterValues.campaignId) {
                        return false
                    }
                }

                return true;
            };

            if ($scope.displayCount && $scope.currentItemType && $scope.showCount!="false") {
                $scope.$watch("ref.parameterValues", function (newValue, oldValue) {
                    // Wait for other watchers to have been executed before running query on server
                    window.setTimeout(function() {
                        if ($scope.validateCondition($scope.ref)) {

                            var condition = $scope.ref;
                            if($scope.ref.parameterValues.propertyValues){
                                $scope.ref.parameterValues.propertyValues = _.map($scope.ref.parameterValues.propertyValues, function (value) {
                                    if(value == undefined || value == null){
                                        return "";
                                    }else {
                                        return value;
                                    }
                                });
                            }
                            if ($scope.currentItemType == 'profile') {
                                var subCondition = $scope.ref;
                                if ($scope.ref.parameterValues.comparisonOperator === 'notContains') {
                                    subCondition = {
                                        "parameterValues": {
                                            "subCondition":
                                                {
                                                    type: "profilePropertyCondition",
                                                    parameterValues: {
                                                        propertyName: $scope.ref.parameterValues.propertyName,
                                                        propertyValue: $scope.ref.parameterValues.propertyValue,
                                                        comparisonOperator: "contains"
                                                    }
                                                }
                                        },
                                        "type": "notCondition"
                                    };
                                } 
                                
                                condition = {
                                    "parameterValues": {
                                        "subConditions": [
                                            {
                                                type: "profilePropertyCondition",
                                                parameterValues: {
                                                    propertyName: "mergedWith",
                                                    comparisonOperator: "missing"
                                                }
                                            },
                                            {
                                                type: "profilePropertyCondition",
                                                parameterValues: {
                                                    propertyName: "systemProperties.isAnonymousProfile",
                                                    comparisonOperator: "missing"
                                                }
                                            },
                                            subCondition
                                        ],
                                        "operator": "and"
                                    },
                                    "type": "booleanCondition"
                                };

                            }

                            queryService.getMatchingItemsCountForCondition($scope.currentItemType, condition).success(function (data) {
                                $scope.matching = data;
                                if($scope.countlabel || $scope.countlabeli18n){
                                    $scope.matching = $scope.getCountLabel() + " "+$scope.matching;
                                } else {
                                    $scope.countClass="subcount";
                                }
                            });
                        } else {
                            $scope.matching = 0;
                            if($scope.countlabel || $scope.countlabeli18n){
                                $scope.matching = $scope.getCountLabel() + " "+$scope.matching;
                            } else {
                                $scope.countClass="subcount";
                            }
                        }
                    });
                }, true);
            }
            //Handle date conditions
            if ($scope.ref.parameterValues){
                if($scope.ref.parameterValues.propertyValuesDate){
                    _.forEach($scope.ref.parameterValues.propertyValuesDate, function (value, index) {
                        $scope.ref.parameterValues.propertyValuesDate[index] = moment(value).toDate();
                    });
                } else if($scope.ref.parameterValues.propertyValueDate) {
                    $scope.ref.parameterValues.propertyValueDate = moment($scope.ref.parameterValues.propertyValueDate).toDate();
                }
            }

            $scope.$watch("types.length", function (newValue, oldValue) {
                console.log("condition.js : condition type list changed, refreshing...");
                $scope.updateTemplate();
            });
            $scope.alternate = !$scope.$parent.alternate;
            $scope.conditionTypes = $scope.$parent.conditionTypes;
            $scope.eventConditionTypes = $scope.$parent.eventConditionTypes;
        }
    };
});
