angular.module('manageScoring', ['ngResource', 'condition', 'wemServices', 'wemDirectives', 'wemModals', 'picker', 'i18n', 'ui.bootstrap', 'datatables', 'dtPager'])
    .run(['$rootScope', 'i18nService', 'genericModal', 'profileService','ruleService','listService', 'scoringService', 'loadingSpinnerService',
        function ($rootScope, i18nService, genericModal, profileService, ruleService, listService, scoringService, loadingSpinnerService) {
            $rootScope.getCurrentModuleResource = function (resourcePath) {
                return ManagersUtils.getCurrentModuleResource(resourcePath);
            };

            $rootScope.tryDeleteItem = function (scoring, callback) {
                // if the scoring we're trying to delete is shared, display a modal to ask confirmation
                if (scoring.scope != ManagersContext.dmfScope) {
                    genericModal.newAlertModal({
                        title: i18nService.format("serverSettings.manageScoring.delete.sharedWarning.title", scoring.name),
                        bodyMessage: i18nService.message("serverSettings.manageScoring.delete.sharedWarning.message")
                    }, function () {
                        deleteScoring(scoring, callback);
                    });
                } else {
                    genericModal.newAlertModal({
                        title: i18nService.format("serverSettings.manageScoring.delete.title", scoring.name),
                        bodyMessage: i18nService.format("serverSettings.manageScoring.delete.message", scoring.name)
                    }, function () {
                        deleteScoring(scoring, callback);
                    });
                }
            };

            var deleteScoring = function(scoring, callback) {
                loadingSpinnerService.show();
                var update = scoringService.buildUpdateBatchRemoveScoring(scoring.id);
                profileService.batchProfilesUpdate(update).success(function () {
                        scoringService.deleteScoring(scoring.id, true).success(function (data) {
                            loadingSpinnerService.hide();
                            ruleService.deleteRule("sendEmailRulePrefix_" + scoring.id);
                            // if data isn't empty, it contains the list of the scoring that use the scoring we're trying to delete as a condition
                            if (data.segments.length > 0 || data.scorings.length > 0) {
                                // display the list of dependent scoring and ask what to do
                                genericModal.newAlertModal({
                                    title: i18nService.format("serverSettings.manageScoring.delete.dependentsModal.title", scoring.name),
                                    bodyTemplateUrl: ManagersContext.localTemplatesPath + "/scoring/scoring-delete-dependents-modal-body.html",
                                    scoringName: scoring.name,
                                    dependents: data
                                }, function () {
                                    loadingSpinnerService.show();
                                    scoringService.deleteScoring(scoring.id, false).success(function (data) {
                                        loadingSpinnerService.hide();
                                        if(callback) {
                                            callback();
                                        }
                                    });
                                });
                            } else {
                                if(callback) {
                                    callback();
                                }
                            }
                        });
                    })
                    .error(function () {
                        //notificationService.error(i18nService.message('serverSettings.manageLists.create.error'));
                        console.error("not updated. Failure!");
                    });
            };
        }])

    .controller('ManageScoringListCtrl', ['$scope', '$modal', 'scoringService', 'queryService', 'listService', '$location', 'DTOptionsBuilder', 'DTColumnBuilder', 'filteringService', 'genericModal', 'i18nService', '$compile',

        function ($scope, $modal, scoringService, queryService, listService, $location, DTOptionsBuilder, DTColumnBuilder, filteringService, genericModal, i18nService, $compile) {
            $scope.items = {};
            $scope.stopPolling = true;

            function serverData(sSource, aoData, fnCallback, oSettings) {
                var draw = _.find(aoData, function (x) {
                    return x.name == "draw"
                }).value;

                var start = _.find(aoData, function (x) {
                    return x.name == "start"
                }).value;

                var length = _.find(aoData, function (x) {
                    return x.name == "length"
                }).value;

                var order = _.map(_.find(aoData, function (x) {
                    return x.name == "order"
                }).value, function (o) {
                    var prop = ($scope.dtColumns[o.column].data ? $scope.dtColumns[o.column].data : $scope.dtColumns[o.column].mData);
                    if (prop == "scoringType") {
                        prop = "metadata.tags";
                    } else {
                        prop = "metadata." + prop;
                    }
                    return prop + ":" + o.dir
                }).join(",");

                scoringService.query(scoringService.constructQuery(length, start, order)).success(function (data) {
                    fnCallback(getResult(draw, data, scoringService));
                    ManagersUtils.initPagerProperties($scope.dtInstance.DataTable, $scope, i18nService);
                }).error(function (data, status, headers, config) {
                    fnCallback(getErrorResult());
                });
            }

            var getResult = function(draw, data, service) {
                angular.forEach(data.list, processItem);

                $scope.stopPolling = false;

                return {
                    "sEcho": draw,
                    "iTotalRecords": data.totalSize,
                    "iTotalDisplayRecords": data.totalSize,
                    "aaData": data.list
                };
            };

            var getErrorResult = function (draw) {
                $scope.searchError = true;
                return {
                    "sEcho": draw,
                    "iTotalRecords": 0,
                    "iTotalDisplayRecords": 0,
                    "aaData": []
                };
            };

            $scope.dtOptions = DTOptionsBuilder.newOptions()
                .withOption('lengthMenu', [ 25, 50, 100 ])
                .withOption('pageLength', 25)
                .withOption('processing', true)
                .withOption('serverSide', true)
                .withOption('sDom', 'lt')
                .withOption('aaSorting', [])
                .withOption('createdRow', createdRow)
                .withFnServerData(serverData)
            ;

            $scope.dtColumns = [
                DTColumnBuilder.newColumn("name").withTitle(i18nService.message("serverSettings.manageScoring.scoringName"))
                    .renderWith(renderNameCol),
                {"data":'scoringType',"title":i18nService.message("serverSettings.manageScoring.type"), "defaultContent": ""},
                {"data":'description',"title":i18nService.message("serverSettings.manageWem.description"), "defaultContent": ""},
                DTColumnBuilder.newColumn(null).withTitle(i18nService.message("serverSettings.manageWem.listAction")).notSortable()
                    .renderWith(actionsHtml).withOption("width","25%").withOption("contentPadding","0")
            ];

            $scope.setDtInstance = function(instance) {
                $scope.dtInstance = instance;
            };

            function createdRow(row, data, dataIndex) {
                console.log("Row is created: "+row);
                // Recompiling so we can bind Angular directive to the DT
                $compile(angular.element(row).contents())($scope);
            }

            function actionsHtml(data, type, full, meta) {
                return "<button title=\"" + i18nService.message("serverSettings.manageScoring.editScoring") + "\" type=\"button\" class=\"btn btn-icon command-edit\" ng-click=\"goToEdit('" + data.id + "')\"><span class=\"md md-edit\"></span></button> " +
                    "<button title=\"" + i18nService.message("serverSettings.manageScoring.manageMembers") + "\" type=\"button\" class=\"btn btn-icon command-profiles\" ng-click=\"goToProfiles('" + data.id + "')\" ><span class=\"md md-profiles\"></span></button>" +
                    "<button title=\"" + i18nService.message("serverSettings.manageWem.delete") + "\" type=\"button\" class=\"btn btn-icon command-delete\" ng-click=\"deleteItem('" + data.id + "')\"><span class=\"md md-delete\"></span></button>";
            }

            $scope.goToEdit = function(id) {
                $scope.stopPolling = true;
                $location.path( "/scoring/" +id );
            };

            $scope.goToProfiles = function(id) {
                $scope.stopPolling = true;
                $location.path( "/scoring/" +id + "/profiles" );
            };

            function renderNameCol(data, type, full, meta) {
                var result = data + ' <span class="badge green-bg">{{items["' + full.id + '"].count}}</span>';
                if(full.isShared) {
                    result += ' <span class="badge  blue-bg">' + i18nService.message("serverSettings.manageScoring.shared") + '</span>'
                }
                if(!full.enabled) {
                    result += ' <span> [' + i18nService.message("serverSettings.manageWem.disabled") + ']</span>'
                }
                return result;
            }

            function getCount (scoringId) {
                var scoringCondition = {
                    "parameterValues": {
                        "subConditions": [
                            {
                                type: "profilePropertyCondition",
                                parameterValues: {
                                    propertyName: "mergedWith",
                                    comparisonOperator: "missing"
                                }
                            },
                            {
                                type: "profilePropertyCondition",
                                parameterValues: {
                                    propertyName: "systemProperties.isAnonymousProfile",
                                    comparisonOperator: "missing"
                                }
                            },
                            {
                                parameterValues: {
                                    propertyName:"scores." + scoringId,
                                    comparisonOperator: "exists"
                                },
                                type: "profilePropertyCondition"
                            },
                            {
                                parameterValues: {
                                    propertyName:"scores." + scoringId,
                                    comparisonOperator: "greaterThan",
                                    propertyValueInteger: 0
                                },
                                type: "profilePropertyCondition"
                            }
                        ],
                        "operator": "and"
                    },
                    "type": "booleanCondition"
                };

                var observable =Rx.Observable.create( function(o){
                    queryService.getMatchingItemsCountForCondition('profile', scoringCondition)
                    .then(function(response){
                        o.onNext(response.data)
                        o.onCompleted();
                    })
                });

                return observable.merge(
                        Rx.Observable
                            .interval(5000)
                            .flatMapLatest(observable)
                            .takeWhile(function (x) { return !$scope.stopPolling; })
                    );
            }

            var processItem = function (itemMetadata) {
                $scope.items[itemMetadata.id] = itemMetadata;
                itemMetadata.isShared = itemMetadata.scope != ManagersContext.dmfScope;
                itemMetadata.count = 0;
                if(_.isEmpty(itemMetadata.tags)){
                    itemMetadata.scoringType = i18nService.message("serverSettings.manageScoring.type.all");
                }else{
                    //tag.demographicProfileProperties.name
                    var type = _.find(itemMetadata.tags, function (tag) {
                        if(tag.indexOf('st:')>=0){
                            return true;
                        }
                    }).split(':')[1];
                    if(type){
                        itemMetadata.scoringType = i18nService.message("tag."+type+".name");
                    }else{
                        itemMetadata.scoringType = "";
                    }
                }
                $scope.items[itemMetadata.id].observable = getCount(itemMetadata.id).subscribe(function(count){
                    $scope.items[itemMetadata.id].count = count;
                });
            };

            $scope.deleteItem = function (id) {
                $scope.tryDeleteItem($scope.items[id], function() {
                    scoringService.deletedItem = id;
                    $scope.dtInstance.dataTable.fnDraw();
                });
            };
        }])

    .controller('ManageScoringCtrl', ['$scope', '$rootScope', '$routeParams', '$resource', '$location', '$route', '$q','profileService', 'ruleService', 'scoringService', 'listService', 'currentNotifRule','currentScoring', 'newScoring', 'definitionService', 'queryService', 'i18nService', 'notificationService', 'loadingSpinnerService',
        function ($scope, $rootScope, $routeParams, $resource, $location, $route, $q, profileService, ruleService, scoringService, listService, currentNotifRule, currentScoring, newScoring, definitionService, queryService, i18nService, notificationService, loadingSpinnerService) {
            $scope.notification = currentNotifRule || {};
            $scope.currentScoring = currentScoring;
            $scope.conditionTypes = [];
            $scope.sharedScoring = currentScoring.metadata.scope != ManagersContext.dmfScope;
            $scope.newScoring = newScoring || false;
            $scope.cannotShareTooltip = i18nService.message('serverSettings.manageScoring.sharedCannotEdit');

            var conditionTypesResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/conditions/tags/:tagId", {tagId: "profileCondition"}, {
                'get': {
                    method: 'GET',
                    isArray: true
                }
            });
            var tagResource = $resource(ManagersContext.proxyServletUrl + "/cxs/definitions/tags/:tagId", {tagId: "profileTags"});

            $scope.tags = {};

            $rootScope.allowedTags = ['aggregated', 'demographic', 'event', 'commerceFactory'];

            $scope.profileTags = tagResource.get({tagId: "profileTags"}, function (value) {
                console.log("manage-scoring.js : tags loaded from Apache Unomi.");

                angular.forEach(value.subTags, function (v) {
                    $scope.tags[v.id] = v;
                });
                definitionService.fillTags("conditionTypes", $scope);
                definitionService.fillTags("eventConditionTypes", $scope);
                $scope.allowedConditionTypes = getAllowedConditions();
            });

            var getAllowedConditions = function () {
                $scope.$watch("conditionTypes.length", function (newVal, oldVal, scope) {
                    if(!$scope.allowedConditionTypes){
                        $scope.allowedConditionTypes = [];
                    }
                    angular.forEach($scope.conditionTypes, function (ct) {
                        if ($rootScope.allowedTags.indexOf(ct.profileTag) !== -1) {
                            ct.name_tr = i18nService.message('condition.' + ct.id + '.name');
                            ct.profileTag_tr = i18nService.message('tag.' + ct.profileTag + '.name');
                            $scope.allowedConditionTypes.push(ct);
                        }
                    });
                });
            };

            $scope.conditionTypes = conditionTypesResource.get({tagId: "profileCondition"}, function (conditionTypes) {
                console.log("manage-scoring.js: Loaded " + conditionTypes.length + " condition types");
            });

            $scope.scoringType = _.find($scope.currentScoring.metadata.tags, function (tag) {
                                    return tag.indexOf('st:') !== -1;
                                });
            /*if(!$scope.scoringType){
                $scope.scoringType = "st:all";
            }*/

            $scope.scoringTypeChange = function (){
                $scope.currentScoring.metadata.tags = _.filter($scope.currentScoring.metadata.tags, function (tag) {
                    return tag && tag.indexOf(':') === -1;
                });
                $scope.currentScoring.metadata.tags.push($scope.scoringType);
            };
            $scope.eventConditionTypes = conditionTypesResource.get({tagId: "usableInPastEventCondition"}, function (conditionTypes) {
                console.log("manage-scoring.js: Loaded " + conditionTypes.length + " condition types");
            });

            scoringService.getImpacted(currentScoring.metadata.id).success(function(data){
                $scope.dependents = data;
            });

            $scope.resetScoring = function ($event) {
                scoringService.getScoring(currentScoring.metadata.id).success(function (scoringData) {
                    currentScoring = scoringData;
                    $route.reload();
                });
            };

            $scope.copyScoring = function ($event) {
                scoringService.scoringToCopy = angular.copy(currentScoring);
                $location.path("/newScoring");
            };

            $scope.hasError = function(propertyName) {
                return ManagersUtils.hasErrors($scope.scoringForm, $scope.scoringForm, propertyName);
            };

            $scope.saveScoring = function (scoringForm) {
                if(scoringForm.$invalid){
                    scoringForm.showErrors = true;

                    var parsedErrors = ManagersUtils.parseErrorsInForm(scoringForm);
                    if(parsedErrors && Object.keys(parsedErrors).length > 0) {
                        ManagersUtils.displayErrors(parsedErrors, i18nService, notificationService);
                        return;
                    }
                }

                var save = function () {
                    var rulePromise = null;
                    if(!$scope.newScoring && !$scope.notification.emailNotifEnabled){
                        rulePromise = ruleService.deleteRule("sendEmailRulePrefix_" + $scope.currentScoring.metadata.id);
                    }
                    if($scope.notification.emailNotifEnabled){
                        rulePromise = ruleService.setRule(scoringService.localToServerScoringRule($scope.currentScoring, $scope.notification.targetEmail, $scope.notification.threshold));
                    } else {
                        if ($scope.newScoring) {
                            scoringService.setScoring($scope.currentScoring, $scope.sharedScoring).success(function () {
                                $location.path("/scoring");
                            });
                        }
                    }
                    if( rulePromise ){
                        rulePromise.then( function () {
                            scoringService.setScoring($scope.currentScoring, $scope.sharedScoring).success(function () {
                                if(!$scope.newScoring && !$scope.notification.emailNotifEnabled) {
                                    var update = scoringService.buildUpdateBatchRemoveScoring($scope.currentScoring.metadata.id);
                                    profileService.batchProfilesUpdate(update);
                                }
                                $location.path("/scoring");
                            });
                        });
                    }
                };

                loadingSpinnerService.show();
                if ($scope.newScoring) {
                    queryService.createWithCallbackIfDoesNotExist('scoring', ManagersContext.dmfScope, $scope.currentScoring.metadata.name, null,
                        "serverSettings.manageScoring.create.alreadyExisting", $scope.currentScoring.metadata.name, save, function() {
                            loadingSpinnerService.hide();
                        });
                } else {
                    save();
                }
            };

            $scope.deleteCurrent = function () {
                var id = $scope.currentScoring.metadata.id;
                var rulePromise = ruleService.deleteRule("sendEmailRulePrefix_" + $scope.currentScoring.metadata.id).success(function(ruleData){
                    var update = scoringService.buildUpdateBatchRemoveScoring($scope.currentScoring.metadata.id);
                    profileService.batchProfilesUpdate(update);
                });
                $q.all([rulePromise]).then(function () {
                    $scope.tryDeleteItem($scope.currentScoring.metadata, function() {
                        scoringService.deletedItem = id;
                        $location.path('/scoring');
                    });
                });
            };

            $scope.deleteScoringElement = function (index) {
                $scope.currentScoring.elements.splice(index, 1);
            };

            $scope.duplicateScoringElement = function (index) {
                $scope.currentScoring.elements.splice(index + 1, 0, angular.copy($scope.currentScoring.elements[index]));
            };

            $scope.addScoringElement = function () {
                if (!$scope.currentScoring.elements) {
                    $scope.currentScoring.elements = [];
                }
                $scope.currentScoring.elements.push({condition: {}, value: 1});
            };
        }]);

